SystemOrganization addCategory: #'RBC-VSA'!

TestCase subclass: #VSAAlertTest
	instanceVariableNames: 'alert'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'RBC-VSA'!

!VSAAlertTest commentStamp: 'rbc 7/28/2011 00:06' prior: 0!
Test alert message methods.
!

!VSAAlertTest methodsFor: 'setup' stamp: 'rbc 11/24/2011 10:13'!
setUp
alert := VSAAlert new.


! !

!VSAAlertTest methodsFor: 'testing' stamp: 'rbc 11/24/2011 10:15'!
testGet
| n |
alert actuality: 1.
alert capability: 1.
n := 'A VSA'.
alert vName: n.
self assert: (alert actuality = 1).
self assert: (alert capability = 1).
self assert: (alert vName = n)
! !

TestCase subclass: #VSABackupRestoreTest
	instanceVariableNames: 'myVSA'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'RBC-VSA'!

!VSABackupRestoreTest commentStamp: 'rbc 1/27/2012 06:38' prior: 0!
Test VSABackupRestore.
!

!VSABackupRestoreTest methodsFor: 'setup' stamp: 'rbc 12/30/2011 23:27'!
setUp
	"Setup the test"
	myVSA := VSA new.
	! !

!VSABackupRestoreTest methodsFor: 'testing' stamp: 'rbc 1/27/2012 08:05'!
testSaveFetch
	"Test saving and fetching VSAs"
	| n a c p testVSA dir fileName |
	
	n := 'Test name'.
	a := 2.
	c := 4.
	p := 8.
	fileName := 'VSABackupRestoreTest'.
	dir := FileDirectory default.
	myVSA myExecutive vName: n.
	myVSA actuality: a.
	myVSA capability: c.
	myVSA potentiality: p.
	VSABackupRestore save: myVSA fileName: fileName.
	testVSA := VSABackupRestore fetch: fileName.
	self assert: (testVSA myExecutive vName = n).
	self assert: (testVSA actuality = a).
	self assert: (testVSA capability = c).
	self assert: (testVSA potentiality = p).
	dir deleteFileNamed: fileName! !

TestCase subclass: #VSAExecutiveMessageTest
	instanceVariableNames: 'message text'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'RBC-VSA'!

!VSAExecutiveMessageTest commentStamp: 'rbc 9/8/2011 11:22' prior: 0!
Testing for policy message methods.
!

!VSAExecutiveMessageTest methodsFor: 'setup' stamp: 'rbc 9/8/2011 11:11'!
setUp
message := VSAPolicyMessage new
! !

!VSAExecutiveMessageTest methodsFor: 'testing' stamp: 'rbc 9/8/2011 11:13'!
testMessage
	"Test setting up a policy message"
	text := 'A test message'.
	message message: text.
	self assert: (message message = text)
! !

TestCase subclass: #VSAExecutiveTest
	instanceVariableNames: 'vExecutive'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'RBC-VSA'!

!VSAExecutiveTest commentStamp: 'rbc 7/28/2011 00:06' prior: 0!
Test executive setters and getters.
!

!VSAExecutiveTest methodsFor: 'setup' stamp: 'rbc 6/4/2011 18:52'!
setUp
vExecutive := VSAExecutive new
! !

!VSAExecutiveTest methodsFor: 'testing' stamp: 'rbc 6/4/2011 18:52'!
testGetName
	vExecutive vName: 'Test'.
	self assert: (vExecutive vName = 'Test')! !

!VSAExecutiveTest methodsFor: 'testing' stamp: 'rbc 6/4/2011 18:53'!
testInterval
	"Test setting and getting the communications interval"
	vExecutive interval: 30.
	self assert: (vExecutive interval = 30)! !

TestCase subclass: #VSAManagerReportTest
	instanceVariableNames: 'report'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'RBC-VSA'!

!VSAManagerReportTest commentStamp: 'rbc 7/28/2011 00:07' prior: 0!
Test manager report setters and getters.
!

!VSAManagerReportTest methodsFor: 'setup' stamp: 'rbc 9/8/2011 11:03'!
setUp
report := VSAManagerReport new.
report actuality: 1.
report capability: 1.
report latency: 1.
report performance: 1
! !

!VSAManagerReportTest methodsFor: 'testing' stamp: 'rbc 9/8/2011 11:04'!
testGet
self assert: (report actuality = 1).
self assert: (report capability = 1).
self assert: (report latency = 1).
self assert: (report performance = 1)! !

TestCase subclass: #VSAManagerTest
	instanceVariableNames: 'manager op op1 op2 msg'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'RBC-VSA'!

!VSAManagerTest commentStamp: 'rbc 7/28/2011 00:08' prior: 0!
This is a detailed test for manager methods.
!

!VSAManagerTest methodsFor: 'setup' stamp: 'rbc 6/4/2011 18:39'!
setUp
	"Setup manager for testing"
	manager := VSAManager new.

! !

!VSAManagerTest methodsFor: 'testing' stamp: 'rbc 7/18/2011 23:00'!
testCostCenter
	"Test profit center calculations"
	manager createOp: VSA.
	manager createOp: VSA.
	manager costCenter: true.
	op1 := manager getOp: 1.
	op2 := manager getOp: 2.
	op1 costCenter: true.
	op2 costCenter: true.
	op1 actuality: 50.
	op2 actuality: 50.
	op1 capability: 25.
	op2 capability: 25.
	op1 potentiality: 5.
	op2 potentiality: 5.
	self assert: (manager latency = (1/5)).
	self assert: (manager performance = (1/10)).
	self assert: (manager productivity = (1/2))
! !

!VSAManagerTest methodsFor: 'testing' stamp: 'rbc 1/28/2012 11:57'!
testEmptyCollection
	"Used to test method handling of an empty operations collection"
	self assert: (manager numberOfOperations = 0).

	"Testing"

	"Sums"
	self assert: (manager actualitySum = nil).
	self assert: (manager capabilitySum = nil).
	self assert: (manager potentialitySum = nil).

	"Mean"
	self assert: (manager actualityMean = nil).
	self assert: (manager capabilityMean = nil).
	self assert: (manager potentialityMean = nil).

	"Achievment"
	self assert: (manager latency = nil).
	self assert: (manager performance = nil).
	self assert: (manager productivity = nil).

	"Actuality"
	self assert: (manager actualityMin = nil).
	self assert: (manager actualityMean = nil).
	self assert: (manager actualityMax = nil).
	
	"Capability"
	self assert: (manager capabilityMin = nil).
	self assert: (manager capabilityMean = nil).
	self assert: (manager capabilityMax = nil).
	
	"Potentiality"
	self assert: (manager potentialityMin = nil).
	self assert: (manager potentialityMean = nil).
	self assert: (manager potentialityMax = nil)
! !

!VSAManagerTest methodsFor: 'testing' stamp: 'rbc 11/29/2011 00:00'!
testGettingAlerts
	"Test getting alerts"
	| c a |

	"Test retrieving an alert"
	manager createOp: VSA.
	op := manager getOp: 1.
	op actuality: 0.
	op capability: 2.
	op potentiality: 4.
	op alertVariance: 0.
	op mark: VSAAlert.
	c := manager getAlerts.
	a := c at: 1.
	self assert: (a isMemberOf: VSAAlert)! !

!VSAManagerTest methodsFor: 'testing' stamp: 'rbc 7/24/2011 09:05'!
testMovingOpsMessages
	"Test moving ops messages"
	manager createOp: VSA.
	manager createOp: VSA.
	op1 := manager getOp: 1.
	op2 := manager getOp: 2.
	msg := VSAOpsMessage new.
	msg to: 2.
	op1 mySupervisor outBox: msg.
	manager supervisorMoveOpsMessages.
	self assert: (op2 mySupervisor inBox isMemberOf: VSAOpsMessage)! !

!VSAManagerTest methodsFor: 'testing' stamp: 'rbc 6/4/2011 18:37'!
testOp
	"Test the creation and accessing of an operation"
	manager createOp: VSA.
	op := manager getOp: 1.
	self assert: (op isMemberOf: VSA).
	self assert: (manager capabilitySum = 1).
	self assert: (manager latency = 1).
	self assert: (manager numberOfOperations = 1).
	self assert: (manager performance = 1).
	self assert: (manager potentialitySum = 1).
	self assert: (manager productivity = 1)! !

!VSAManagerTest methodsFor: 'testing' stamp: 'rbc 1/28/2012 11:56'!
testOperationsDoNotCompare
	"Test nil throwing when operations do not compare"
	
	"Test setup"
	manager createOp: VSA.
	manager createOp: VSA.
	op1 := manager getOp: 1.
	op2 := manager getOp: 2.
	op1 actuality: 4.
	op2 actuality: 8.
	op1 capability: 10.
	op2 capability: 12.
	op1 potentiality: 14.
	op2 potentiality: 16.
	manager operationsCompare: false.

	"Testing"

	"Sums"
	self assert: (manager actualitySum = nil).
	self assert: (manager capabilitySum = nil).
	self assert: (manager potentialitySum = nil).

	"Mean"
	self assert: (manager actualityMean = nil).
	self assert: (manager capabilityMean = nil).
	self assert: (manager potentialityMean = nil).

	"Achievment"
	self assert: (manager latency = nil).
	self assert: (manager performance = nil).
	self assert: (manager productivity = nil).

	"Actuality"
	self assert: (manager actualityMin = nil).
	self assert: (manager actualityMean = nil).
	self assert: (manager actualityMax = nil).
	
	"Capability"
	self assert: (manager capabilityMin = nil).
	self assert: (manager capabilityMean = nil).
	self assert: (manager capabilityMax = nil).
	
	"Potentiality"
	self assert: (manager potentialityMin = nil).
	self assert: (manager potentialityMean = nil).
	self assert: (manager potentialityMax = nil)! !

!VSAManagerTest methodsFor: 'testing' stamp: 'rbc 7/18/2011 22:35'!
testProfitCenter
	"Test profit center calculations"
	manager createOp: VSA.
	manager createOp: VSA.
	op1 := manager getOp: 1.
	op2 := manager getOp: 2.
	op1 actuality: 20.
	op2 actuality: 20.
	op1 capability: 30.
	op2 capability: 30.
	op1 potentiality: 50.
	op2 potentiality: 50.
	self assert: (manager latency = (3/5)).
	self assert: (manager performance = (2/5)).
	self assert: (manager productivity = (2/3))
! !

!VSAManagerTest methodsFor: 'testing' stamp: 'rbc 6/4/2011 18:41'!
testReport
	"Test the creation and sending of a report"
	manager createOp: VSA.
	op := manager getOp: 1.
	manager sendManagerReport: VSAManagerReport.
	msg := manager outBox.
	self assert: (msg isMemberOf: VSAManagerReport)
! !

!VSAManagerTest methodsFor: 'testing' stamp: 'rbc 7/23/2011 23:49'!
testSendingOpsMessage
	"Test the creation and sending of an ops message"
	| m o |
	m := VSAOpsMessage new.
	m to: 1.
	manager createOp: VSA.
	o := manager getOp: 1.
	manager sendOpsMessage: m.
	self assert: (o mySupervisor inBox isMemberOf: VSAOpsMessage)
! !

!VSAManagerTest methodsFor: 'testing' stamp: 'rbc 1/28/2012 11:59'!
testStats
	"Test statistics"
	manager createOp: VSA.
	manager createOp: VSA.
	op1 := manager getOp: 1.
	op2 := manager getOp: 2.
	op1 actuality: 4.
	op2 actuality: 8.
	op1 capability: 10.
	op2 capability: 12.
	op1 potentiality: 14.
	op2 potentiality: 16.
	self assert: (manager actualityMin = 4).
	self assert: (manager actualityMean = 6).
	self assert: (manager actualityMax = 8).
	self assert: (manager capabilityMin = 10).
	self assert: (manager capabilityMean = 11).
	self assert: (manager capabilityMax = 12).
	self assert: (manager potentialityMin = 14).
	self assert: (manager potentialityMean = 15).
	self assert: (manager potentialityMax = 16)! !

TestCase subclass: #VSAMessageTest
	instanceVariableNames: 'myMessage'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'RBC-VSA'!

!VSAMessageTest commentStamp: 'rbc 11/25/2011 20:34' prior: 0!
Testing for basic VSA message methods.
!

!VSAMessageTest methodsFor: 'setup' stamp: 'rbc 11/24/2011 09:29'!
setUp
myMessage := VSAMessage new.! !

!VSAMessageTest methodsFor: 'testing' stamp: 'rbc 11/24/2011 10:17'!
testGets
| m n |
m := 'Some message text'.
n := 'A VSA name'.
myMessage setTimeStamp.
myMessage message: m.
myMessage vName: n.
self assert: (myMessage message = m).
self assert: (myMessage timeStamp isMemberOf: Array ).
self assert: (myMessage vName = n).
! !

TestCase subclass: #VSAPlannerMessageTest
	instanceVariableNames: 'message'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'RBC-VSA'!

!VSAPlannerMessageTest commentStamp: 'rbc 9/8/2011 11:21' prior: 0!
Testing for planning message methods.!

!VSAPlannerMessageTest methodsFor: 'setup' stamp: 'rbc 9/8/2011 11:13'!
setUp
message := VSAPlannerMessage new
! !

!VSAPlannerMessageTest methodsFor: 'testing' stamp: 'rbc 9/8/2011 11:14'!
testGet
	"Test setting up a planner message"
	message potentiality: 1.
	self assert: (message potentiality = 1)! !

TestCase subclass: #VSATest
	instanceVariableNames: 'myVSA'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'RBC-VSA'!

!VSATest commentStamp: 'rbc 7/30/2011 10:26' prior: 0!
An end to end test for the VSA class, along with component classes.
!

!VSATest methodsFor: 'setup' stamp: 'rbc 6/3/2011 23:06'!
setUp
	"Setup test environment"
	myVSA := VSA new

! !

!VSATest methodsFor: 'testing' stamp: 'rbc 7/18/2011 21:43'!
testCostCenter
	"Test cost center calculations"
	myVSA costCenter: true.
	myVSA actuality: 100.
	myVSA capability: 50.
	myVSA potentiality: 10.
	self assert: (myVSA latency = (1/5)).
	self assert: (myVSA performance = (1/10)).
	self assert: (myVSA productivity = (1/2))! !

!VSATest methodsFor: 'testing' stamp: 'rbc 12/30/2011 23:21'!
testGets
	"Test out get methods"
	myVSA actuality: 0.
	myVSA capability: 2.
	myVSA potentiality: 4.
	myVSA alertVariance: 0.
	myVSA mark: VSAAlert.
	self assert: (myVSA actuality = 0).
	self assert: (myVSA capability = 2).
	self assert: (myVSA potentiality = 4).
	self assert: (myVSA alertVariance = 0).
	self assert: (myVSA alert isMemberOf: VSAAlert).
	self assert: (myVSA myExecutive isMemberOf: VSAExecutive).
	self assert: (myVSA myPlanner isMemberOf: VSAPlanner).
	self assert: (myVSA myManager isMemberOf: VSAManager).
	self assert: (myVSA costCenter = false).
	self assert: (myVSA myManager costCenter = false).
	self assert: (myVSA operationsCompare = true).
	myVSA costCenter: true.
	self assert: (myVSA costCenter = true).
	self assert: (myVSA myManager costCenter = true)! !

!VSATest methodsFor: 'testing' stamp: 'rbc 11/24/2011 10:12'!
testMessaging
	"Test message sends"
	| msg pol t |
	myVSA myManager createOp: VSA.

	"Test the managers report"
	myVSA myManager sendManagerReport: VSAManagerReport.
	myVSA moveMessages.
	msg := myVSA myPlanner inBox.
	self assert: (msg isMemberOf: VSAManagerReport).
	msg := myVSA myExecutive inBox.
	self assert: (msg isMemberOf: VSAManagerReport).

	"Test sending a planner message"
	msg := VSAPlannerMessage new.
	myVSA myPlanner outBox: msg.
	myVSA moveMessages.
	t := myVSA myManager inBox.
	self assert: (t isMemberOf: VSAPlannerMessage).
	t := myVSA myExecutive inBox.
	self assert: (t isMemberOf: VSAPlannerMessage).

	"Test sending policies"
	pol := VSAPolicyMessage new.
	myVSA myExecutive outBox: pol.
	myVSA moveMessages.
	t := myVSA myPlanner inBox.
	self assert: (t isMemberOf: VSAPolicyMessage).
	t := myVSA myManager inBox.
	self assert: (t isMemberOf: VSAPolicyMessage)! !

!VSATest methodsFor: 'testing' stamp: 'rbc 1/18/2012 07:38'!
testProcesses
	"Test running processes"
	myVSA myExecutive interval: 1.
	myVSA startProcesses.
	myVSA runProcesses: false.
	self assert: (myVSA runProcesses = false)
	! !

!VSATest methodsFor: 'testing' stamp: 'rbc 7/18/2011 21:06'!
testProfitCenter
	"Test profit center calculations"
	myVSA actuality: 40.
	myVSA capability: 60.
	myVSA potentiality: 100.
	self assert: (myVSA latency = (3/5)).
	self assert: (myVSA performance = (2/5)).
	self assert: (myVSA productivity = (2/3)).
! !

!VSATest methodsFor: 'testing' stamp: 'rbc 9/24/2011 17:58'!
testUpdates
	"Test updates from subordinate operations"
	myVSA myManager createOp: VSA.
	myVSA update.
	self assert: (myVSA actuality = 1).
	self assert: (myVSA capability = 1).
	self assert: (myVSA potentiality = 1).
	self assert: (myVSA lastUpdate isMemberOf: Array )
! !

Object subclass: #VSA
	instanceVariableNames: 'mySupervisor myExecutive myPlanner myManager actuality capability potentiality alerts alertVariance runProcesses costCenter operationsCompare lastUpdate'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'RBC-VSA'!

!VSA commentStamp: 'rbc 7/28/2011 12:51' prior: 0!
Both a viable system and operation (system 1).  Each VSA starts an Executive (5),
Planner (4) and Manager (3).  A VSA is also paired with a Supervisor (2) in its role as
an operation.

Viable System Terminology:

1) Operation(s) - A primary system activity
2) Supervisor(s) - Coordinates operations
3) Manager - Plans and controls current operations
4) Planner - Monitors environment and plans for the future
5) Executive - Maintains system identity

Operation Description:

-A primary activity that performs part of the transformation of the viable system
-Generates algedonic alerts that are sent to Manager
-Responds to audit messages from Manager
-Communicates with the paired Supervisor

References:

-1972, Stafford Beer, Brain of the Firm; Allen Lane, The Penguin Press, London, Herder and Herder, USA.
-1979, Stafford Beer, The Heart of Enterprise; John Wiley, London and New York.

Copyright 2011 Robert Bruce Carleton. All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are
permitted provided that the following conditions are met:

   1. Redistributions of source code must retain the above copyright notice, this list of
      conditions and the following disclaimer.

   2. Redistributions in binary form must reproduce the above copyright notice, this list
      of conditions and the following disclaimer in the documentation and/or other materials
      provided with the distribution.

THIS SOFTWARE IS PROVIDED BY ROBERT BRUCE CARLETON ''AS IS'' AND ANY EXPRESS OR IMPLIED
WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL ROBERT BRUCE CARLETON OR
CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

The views and conclusions contained in the software and documentation are those of the
authors and should not be interpreted as representing official policies, either expressed
or implied, of Robert Bruce Carleton.
!

!VSA class methodsFor: 'instance creation' stamp: 'rbc 5/17/2011 20:35'!
new
	"Set up the VSA and it's systems"
	^super new createVSA! !

!VSA methodsFor: 'accessing' stamp: 'rbc 5/28/2011 09:44'!
actuality
	"Get the actuality."
	^ actuality
	! !

!VSA methodsFor: 'accessing' stamp: 'rbc 5/28/2011 09:44'!
actuality: aNumber
	"Set the actuality."
	actuality := aNumber
	! !

!VSA methodsFor: 'accessing' stamp: 'rbc 5/28/2011 17:33'!
alert
	"Get an alert from the alerts queue"
	^alerts nextOrNil! !

!VSA methodsFor: 'accessing' stamp: 'rbc 5/28/2011 13:33'!
alert: anObject
	"Add an alert to the alerts queue"
	alerts nextPut: anObject! !

!VSA methodsFor: 'accessing' stamp: 'rbc 5/28/2011 09:45'!
alertVariance
	"Get the alert variance"
	^alertVariance.
	! !

!VSA methodsFor: 'accessing' stamp: 'rbc 5/30/2011 12:12'!
alertVariance: aNumber
	"Set the alert variance.  This specifies the threshold for creating an alert
	when actuality deviates from capability by aNumber.  The mark: method
	uses this parameter when it tests productivity."
	alertVariance := aNumber.
	! !

!VSA methodsFor: 'accessing' stamp: 'rbc 5/28/2011 13:40'!
capability
	"Get the capability."
	^ capability
	! !

!VSA methodsFor: 'accessing' stamp: 'rbc 5/28/2011 13:40'!
capability: aNumber
	"Set the capability of the VSA."
	capability := aNumber
	! !

!VSA methodsFor: 'accessing' stamp: 'rbc 7/17/2011 16:09'!
costCenter
	"Get cost center boolean.  If it's not a profit center, it's a cost center"
	^costCenter
	! !

!VSA methodsFor: 'accessing' stamp: 'rbc 7/21/2011 23:15'!
costCenter: aBoolean
	"Set the cost center boolean. With a profit center, more is better.
	With a cost center less is better.  This boolean is used to determine
	how latency, performance and productivity are calculated.	See the
	end of chapter 11 in: 1972, Stafford Beer, Brain of the Firm"

	costCenter := aBoolean.
	myManager costCenter: aBoolean.
	! !

!VSA methodsFor: 'initialization' stamp: 'rbc 7/26/2011 20:18'!
createVSA
	"Create the VSA.  Create an alerts queue, set up the intial environment"
	self mySupervisor: VSASupervisor.
	self myExecutive: VSAExecutive.
	self myPlanner: VSAPlanner.
	self myManager: VSAManager.
	alerts := SharedQueue new.
	self actuality: 1.
	self capability: 1.
	self potentiality: 1.
	self alertVariance: 1.
	self runProcesses: false.
	self costCenter: false.
	self operationsCompare: true! !

!VSA methodsFor: 'updating' stamp: 'rbc 9/24/2011 17:47'!
lastUpdate
	"Get the time of the last update"
	^lastUpdate! !

!VSA methodsFor: 'calculations' stamp: 'rbc 7/17/2011 21:39'!
latency
	"Calculate the latency."
	costCenter
	ifFalse: [^ capability / potentiality]
	ifTrue: [^ potentiality / capability]
	! !

!VSA methodsFor: 'events-triggering' stamp: 'rbc 11/25/2011 20:46'!
mark: anObject
	| a |
	"Mark an alert reporting threshold, generate an alert from the specified object if needed"
	actuality > (capability + alertVariance) ifTrue: [a := anObject new.
										a vName: myExecutive vName.
										a actuality: actuality.
										a capability: capability.
										self alert: a].
	actuality < (capability - alertVariance) ifTrue: [a := anObject new.	
										a vName: myExecutive vName.
										a actuality: actuality.
										a capability: capability.
										self alert: a].
! !

!VSA methodsFor: 'message handling' stamp: 'rbc 12/23/2011 11:30'!
moveMessages
	"Move messages."
	| msg policy |

	"Move messages between operations."
	myManager supervisorMoveOpsMessages.
	
	"Send managers report to both planner and executive."
	[myManager outBoxSize > 0]
			whileTrue: [	msg := myManager outBox.
				myPlanner inBox: msg.
				myExecutive inBox: msg].
			
	"Send planners report to both manager and executive."
	[myPlanner outBoxSize > 0]
			whileTrue: [	msg := myPlanner outBox.
				myManager inBox: msg.
				myExecutive inBox: msg].
	
	"Send executive policy messages."
	[myExecutive outBoxSize > 0]
			whileTrue: [	msg := myExecutive outBox.
				myManager inBox: msg.
				myPlanner inBox: msg]

! !

!VSA methodsFor: 'accessing' stamp: 'rbc 5/17/2011 20:37'!
myExecutive
	"Return the executive"
	^myExecutive
	! !

!VSA methodsFor: 'accessing' stamp: 'rbc 5/17/2011 20:33'!
myExecutive: anObject
	"Create the executive"
	myExecutive := anObject new.
	! !

!VSA methodsFor: 'accessing' stamp: 'rbc 5/17/2011 20:40'!
myManager
	"Return the manager"
	^myManager
	! !

!VSA methodsFor: 'accessing' stamp: 'rbc 5/17/2011 20:39'!
myManager: anObject
	"Create the manager"
	myManager := anObject new.
	! !

!VSA methodsFor: 'accessing' stamp: 'rbc 5/17/2011 20:39'!
myPlanner
	"Return the planner"
	^myPlanner
	! !

!VSA methodsFor: 'accessing' stamp: 'rbc 5/17/2011 20:39'!
myPlanner: anObject
	"Create the planner"
	myPlanner := anObject new.
	! !

!VSA methodsFor: 'accessing' stamp: 'rbc 7/12/2011 17:19'!
mySupervisor
	"Return the supervisor"
	^mySupervisor
	! !

!VSA methodsFor: 'accessing' stamp: 'rbc 7/12/2011 17:19'!
mySupervisor: anObject
	"Create the supervisor"
	mySupervisor := anObject new.
	! !

!VSA methodsFor: 'accessing' stamp: 'rbc 11/29/2011 00:51'!
numberOfAlerts
	"Get the number of alerts"
	^alerts size! !

!VSA methodsFor: 'accessing' stamp: 'rbc 7/26/2011 20:22'!
operationsCompare
	"Do operations have comparable achievement numbers?"
	^operationsCompare
	! !

!VSA methodsFor: 'accessing' stamp: 'rbc 1/27/2012 18:38'!
operationsCompare: aBoolean
	"Do operations have comparable achievement numbers?
	This is used to control updates."
	operationsCompare := aBoolean.
	myManager operationsCompare: aBoolean
! !

!VSA methodsFor: 'calculations' stamp: 'rbc 7/17/2011 21:41'!
performance
	"Calculate performance"
	costCenter
	ifFalse: [^actuality / potentiality]
	ifTrue: [^potentiality / actuality]
	! !

!VSA methodsFor: 'accessing' stamp: 'rbc 5/28/2011 09:46'!
potentiality
	"Get the potentiality."
	^ potentiality
	! !

!VSA methodsFor: 'accessing' stamp: 'rbc 5/28/2011 09:47'!
potentiality: aNumber
	"Set the potentiality."
	potentiality := aNumber
	! !

!VSA methodsFor: 'calculations' stamp: 'rbc 7/17/2011 21:41'!
productivity
	"Calculate productivity."
	costCenter
	ifFalse: [^ actuality / capability]
	ifTrue: [^ capability / actuality]
	! !

!VSA methodsFor: 'accessing' stamp: 'rbc 5/30/2011 22:00'!
runProcesses
	"Get the intended process run state"
	^runProcesses.
	! !

!VSA methodsFor: 'accessing' stamp: 'rbc 5/30/2011 22:00'!
runProcesses: aBoolean
	"Set the intended process run state"
	runProcesses := aBoolean.
	! !

!VSA methodsFor: 'updating' stamp: 'rbc 9/24/2011 17:47'!
setLastUpdate
	"Set the time of the last update"
	lastUpdate := Date dateAndTimeNow! !

!VSA methodsFor: 'initialization' stamp: 'rbc 7/24/2011 18:00'!
startProcesses
	"Start VSA processes."
	self runProcesses: true.
	[[runProcesses] whileTrue:
		[self update.
		self mark: VSAAlert.
		self moveMessages.
		(Delay forSeconds: (myExecutive interval)) wait]] fork.
! !

!VSA methodsFor: 'updating' stamp: 'rbc 9/24/2011 17:48'!
update
	"Update actuality, capability and potentiality
	by extracting it from subordinate operations.
	This may not be desired if the subordinate
	operations do not have comparable achivement
	numbers.  Use operationsCompare to control updates.
	It is assumed that some other means of determining
	achivement will be provided, if subordinate operations do
	not provide comparable numbers."
	| b |
	b := myManager numberOfOperations.
	(((b = 1) | (b > 1)) & operationsCompare)
	ifTrue:
		[self actuality: myManager actualitySum.
		self capability: myManager capabilitySum.
		self potentiality: myManager potentialitySum.
		self setLastUpdate].
! !

Object subclass: #VSABackupRestore
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'RBC-VSA'!

!VSABackupRestore commentStamp: 'rbc 7/27/2011 23:52' prior: 0!
Used to store and retrieve VSAs from disk.
!

!VSABackupRestore class methodsFor: 'objects from disk' stamp: 'rbc 5/25/2011 22:49'!
fetch: aString
	"Restore a VSA from file"
	| rr v |
	rr := ReferenceStream fileNamed: aString.
	v := rr next.
	rr close.
	^v! !

!VSABackupRestore class methodsFor: 'objects from disk' stamp: 'rbc 6/11/2011 16:32'!
save: anObject fileName: aString
	"Save an entire VSA object to disk"
	| rr |
	rr := ReferenceStream fileNamed: aString.
	rr nextPut: anObject.
	rr close.! !

Object subclass: #VSAExecutive
	instanceVariableNames: 'vName interval inBox outBox'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'RBC-VSA'!

!VSAExecutive commentStamp: 'rbc 7/28/2011 08:14' prior: 0!
A VSA executive class, or system 5.

-Policy decisions within the system as a whole
-Balancing demands from different parts of the system
-Steering the system as a whole
!

!VSAExecutive class methodsFor: 'instance creation' stamp: 'rbc 5/30/2011 19:01'!
new
	"Set up the executive"
	^super new createExecutive.
	! !

!VSAExecutive methodsFor: 'initialization' stamp: 'rbc 8/29/2011 07:22'!
createExecutive
	"Initialize the executive"
	self interval: 60.
	inBox := SharedQueue new.
	outBox := SharedQueue new
! !

!VSAExecutive methodsFor: 'accessing' stamp: 'rbc 8/29/2011 07:20'!
inBox
	"Read a message from the in box"
	^inBox nextOrNil! !

!VSAExecutive methodsFor: 'accessing' stamp: 'rbc 8/29/2011 07:21'!
inBox: anObject
	"Add a message object to the in box"
	inBox nextPut: anObject! !

!VSAExecutive methodsFor: 'accessing' stamp: 'rbc 5/30/2011 18:56'!
interval
	"Get the interval for for system activity, measured in seconds"
	^interval.
	! !

!VSAExecutive methodsFor: 'accessing' stamp: 'rbc 5/30/2011 18:56'!
interval: aNumber
	"Set the interval for for system activity, measured in seconds"
	interval := aNumber.
	! !

!VSAExecutive methodsFor: 'accessing' stamp: 'rbc 8/29/2011 07:21'!
outBox
	"Read a message from the out box"
	^outBox nextOrNil! !

!VSAExecutive methodsFor: 'accessing' stamp: 'rbc 8/29/2011 07:21'!
outBox: anObject
	"Add a message object to the out box"
	outBox nextPut: anObject
! !

!VSAExecutive methodsFor: 'accessing' stamp: 'rbc 12/23/2011 11:17'!
outBoxSize
	"Determine the size of the out box"
	^outBox size
! !

!VSAExecutive methodsFor: 'accessing' stamp: 'rbc 5/14/2011 23:00'!
vName
	"Get the VSA Name"
	^vName
	! !

!VSAExecutive methodsFor: 'accessing' stamp: 'rbc 5/14/2011 22:59'!
vName: aString
	"Set the VSA Name"
	vName := aString
	! !

Object subclass: #VSAManager
	instanceVariableNames: 'opsCollection inBox outBox costCenter operationsCompare'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'RBC-VSA'!

!VSAManager commentStamp: 'rbc 7/28/2011 08:13' prior: 0!
A VSA manager class, or system 3.

-Establishes the rules, resources and responsibilities of operations
-Communicates with Planner and Executive
-Sends audit messages to operations
-Accepts algedonic alerts from operations
!

!VSAManager class methodsFor: 'instance creation' stamp: 'rbc 5/30/2011 16:23'!
new
	"Set up the Manager"
	^super new createManager.
	! !

!VSAManager methodsFor: 'calculations' stamp: 'rbc 1/27/2012 18:28'!
actualityMax
	"Calculate maximum actuality of all operations."
	| max n a |
	n := self numberOfOperations.
	(n > 0 & operationsCompare)
	ifTrue: [
		max := (opsCollection at: 1) actuality.
		opsCollection do: [:o | a := o actuality.
			max < a 
				ifTrue: [max := a]
			].
		^max]
	ifFalse: [^nil]! !

!VSAManager methodsFor: 'calculations' stamp: 'rbc 1/27/2012 18:29'!
actualityMean
	"Calculate the mean actuality for all operations"
	| n s |
	n := self numberOfOperations.
	(n > 0 & operationsCompare)
	ifTrue: [s := self actualitySum. ^s / n]
	ifFalse: [^nil]! !

!VSAManager methodsFor: 'calculations' stamp: 'rbc 1/27/2012 18:29'!
actualityMin
	"Calculate minimum actuality of all operations."
	| min n a |
	n := self numberOfOperations.
	(n > 0 & operationsCompare)
	ifTrue: [
		min := (opsCollection at: 1) actuality.
		opsCollection do: [:o | a := o actuality.
			min > a 
				ifTrue: [min := a]
			].
		^min]
	ifFalse: [^nil]! !

!VSAManager methodsFor: 'calculations' stamp: 'rbc 1/27/2012 18:29'!
actualitySum
	"Calculate summed actuality of all operations."
	| s n |
	s := 0.
	n := self numberOfOperations.
	(n > 0 & operationsCompare)
	ifTrue: [opsCollection do: [:a | s := s + a actuality]. ^s]
	ifFalse: [^nil]! !

!VSAManager methodsFor: 'calculations' stamp: 'rbc 1/27/2012 18:29'!
capabilityMax
	"Calculate maximum capability of all operations."
	| max n c |
	n := self numberOfOperations.
	(n > 0 & operationsCompare)
	ifTrue: [
		max := (opsCollection at: 1) capability.
		opsCollection do: [:o | c := o capability.
			max < c 
				ifTrue: [max := c]
			].
		^max]
	ifFalse: [^nil]! !

!VSAManager methodsFor: 'calculations' stamp: 'rbc 1/27/2012 18:29'!
capabilityMean
	"Calculate the mean capability for all operations"
	| n s |
	n := self numberOfOperations.
	(n > 0 & operationsCompare)
	ifTrue: [s := self capabilitySum. ^s / n]
	ifFalse: [^nil]! !

!VSAManager methodsFor: 'calculations' stamp: 'rbc 1/27/2012 18:30'!
capabilityMin
	"Calculate minimum capability of all operations."
	| min n c |
	n := self numberOfOperations.
	(n > 0 & operationsCompare)
	ifTrue: [
		min := (opsCollection at: 1) capability.
		opsCollection do: [:o | c := o capability.
			min > c 
				ifTrue: [min := c]
			].
		^min]
	ifFalse: [^nil]! !

!VSAManager methodsFor: 'calculations' stamp: 'rbc 1/27/2012 18:30'!
capabilitySum
	"Calculate summed capability of all operations."
	| s n |
	s := 0.
	n := self numberOfOperations.
	(n > 0 & operationsCompare)
	ifTrue: [opsCollection do: [:a | s := s + a capability].	^s]
	ifFalse: [^nil]
	! !

!VSAManager methodsFor: 'accessing' stamp: 'rbc 7/17/2011 16:12'!
costCenter
	"Get cost center boolean.  If it's not a profit center, it's a cost center"
	^costCenter
	! !

!VSAManager methodsFor: 'accessing' stamp: 'rbc 7/17/2011 16:12'!
costCenter: aBoolean
	"Set cost center boolean.  If it's not a profit center, it's a cost center"
	costCenter := aBoolean
	! !

!VSAManager methodsFor: 'initialization' stamp: 'rbc 1/27/2012 18:11'!
createManager
	"Manager startup"
	inBox := SharedQueue new.
	outBox := SharedQueue new.
	costCenter := false.
	self operationsCompare: true! !

!VSAManager methodsFor: 'accessing' stamp: 'rbc 11/29/2011 00:05'!
createOp: anObject
	"Create an operation."
	| c |
	c := opsCollection isKindOf: OrderedCollection.
	c ifFalse: [opsCollection := OrderedCollection new].
	opsCollection add: anObject new
! !

!VSAManager methodsFor: 'accessing' stamp: 'rbc 11/29/2011 20:03'!
getAlerts
	"Retrieve and return alerts from subordinate operations"
	| a n c |
	c := OrderedCollection new.
	n := self numberOfOperations.
	n > 0
	ifTrue: [opsCollection do:
		[:o | [o numberOfAlerts > 0]
			whileTrue: [a := o alert. c add: a]
		]
	].
	^c! !

!VSAManager methodsFor: 'accessing' stamp: 'rbc 1/31/2010 13:58'!
getOp: aNumber
	"Get an operation."
	| op |
	op := opsCollection at: aNumber.
	^ op! !

!VSAManager methodsFor: 'accessing' stamp: 'rbc 5/30/2011 16:55'!
inBox
	"Read a message from the in box"
	^inBox nextOrNil.
	
	! !

!VSAManager methodsFor: 'accessing' stamp: 'rbc 5/30/2011 16:55'!
inBox: anObject
	"Add a message object to the in box"
	inBox nextPut: anObject
	! !

!VSAManager methodsFor: 'calculations' stamp: 'rbc 1/27/2012 18:31'!
latency
	"Calculate aggregate latency of all operations."
	| c p n |
	c := self capabilitySum.
	p := self potentialitySum.
	n := self numberOfOperations.
	(n > 0 & operationsCompare)
	ifTrue: [ costCenter
		ifFalse: [^c/p]
		ifTrue: [^p/c]
		]
	ifFalse: [^nil]! !

!VSAManager methodsFor: 'accessing' stamp: 'rbc 5/30/2011 21:44'!
numberOfOperations
	"Get the number of operations."
	| c |
	c := opsCollection isKindOf: OrderedCollection.
	c
	ifFalse: [^0]
	ifTrue: [^opsCollection size].
! !

!VSAManager methodsFor: 'accessing' stamp: 'rbc 1/27/2012 18:09'!
operationsCompare
	"Do operations have comparable achievement numbers?"
	^operationsCompare
	! !

!VSAManager methodsFor: 'accessing' stamp: 'rbc 1/27/2012 18:10'!
operationsCompare: aBoolean
	"Do operations have comparable achievement numbers?
	This is used to control calculations."
	operationsCompare := aBoolean
	! !

!VSAManager methodsFor: 'accessing' stamp: 'rbc 5/30/2011 16:53'!
outBox
	"Read a message from the out box"
	^outBox nextOrNil.
	
	! !

!VSAManager methodsFor: 'accessing' stamp: 'rbc 5/30/2011 16:53'!
outBox: anObject
	"Add a message object to the out box"
	outBox nextPut: anObject
	! !

!VSAManager methodsFor: 'accessing' stamp: 'rbc 12/23/2011 11:18'!
outBoxSize
	"Determine the size of the out box"
	^outBox size
! !

!VSAManager methodsFor: 'calculations' stamp: 'rbc 1/27/2012 18:31'!
performance
	"Calculate aggregate performance of all operations."
	| a p n |
	a := self actualitySum.
	p := self potentialitySum.
	n := self numberOfOperations.
	(n > 0 & operationsCompare)
	ifTrue: [ costCenter
		ifFalse: [^ a / p]
		ifTrue: [^ p / a]
		]
	ifFalse: [^nil]
! !

!VSAManager methodsFor: 'calculations' stamp: 'rbc 1/27/2012 18:31'!
potentialityMax
	"Calculate maximum potentiality of all operations."
	| max n p |
	n := self numberOfOperations.
	(n > 0 & operationsCompare)
	ifTrue: [
		max := (opsCollection at: 1) potentiality.
		opsCollection do: [:o | p := o potentiality.
			max < p 
				ifTrue: [max := p]
			].
		^max]
	ifFalse: [^nil]! !

!VSAManager methodsFor: 'calculations' stamp: 'rbc 1/27/2012 18:31'!
potentialityMean
	"Calculate the mean potentiality for all operations"
	| n s |
	n := self numberOfOperations.
	(n > 0 & operationsCompare)
	ifTrue: [s := self potentialitySum. ^s / n]
	ifFalse: [^nil]! !

!VSAManager methodsFor: 'calculations' stamp: 'rbc 1/27/2012 18:32'!
potentialityMin
	"Calculate minimum potentiality of all operations."
	| min n p |
	n := self numberOfOperations.
	(n > 0 & operationsCompare)
	ifTrue: [
		min := (opsCollection at: 1) potentiality.
		opsCollection do: [:o | p := o potentiality.
			min > p 
				ifTrue: [min := p]
			].
		^min]
	ifFalse: [^nil]! !

!VSAManager methodsFor: 'calculations' stamp: 'rbc 1/27/2012 18:32'!
potentialitySum
	"Calculate summed potentiality of all operations."
	| p n |
	p := 0.
	n := self numberOfOperations.
	(n > 0 & operationsCompare)
	ifTrue: [opsCollection do: [:a | p := p + a potentiality]. ^p]
	ifFalse: [^nil]! !

!VSAManager methodsFor: 'calculations' stamp: 'rbc 1/27/2012 18:32'!
productivity
	"Calculate aggregate productivity of all operations."
	| a c n |
	a := self actualitySum.
	c := self capabilitySum.
	n := self numberOfOperations.
	(n > 0 & operationsCompare)
	ifTrue: [ costCenter
		ifFalse: [^a/c]
		ifTrue: [^c/a]
		]
	ifFalse: [^nil]
! !

!VSAManager methodsFor: 'message handling' stamp: 'rbc 11/29/2011 00:19'!
sendManagerReport: anObject
	"Send manager report to planner"
	| a c l m p report |
	a := self actualitySum.
	c := self capabilitySum.
	l := self latency.
	m := self getAlerts.
	p := self performance.
	report := anObject new.
	report actuality: a.
	report capability: c.
	report latency: l.
	report performance: p.
	report message: m.
	self outBox: report.
	 ! !

!VSAManager methodsFor: 'message handling' stamp: 'rbc 7/24/2011 09:12'!
sendOpsMessage: aMessage
	"Send a message to an operation"
	| to op |
	to := aMessage to.
	op := self getOp: to.
	op mySupervisor inBox: aMessage! !

!VSAManager methodsFor: 'message handling' stamp: 'rbc 12/17/2011 00:59'!
supervisorMoveOpsMessages
	"Move ops messages from one operation to another"
	| n msg |
	n := self numberOfOperations.
	n > 0
	ifTrue: [
		opsCollection do: [ :op | [op mySupervisor outBoxSize > 0]
			whileTrue: [msg := op mySupervisor outBox.
						self sendOpsMessage: msg]
		]
	]
! !

Object subclass: #VSAMessage
	instanceVariableNames: 'timeStamp message vName'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'RBC-VSA'!

!VSAMessage commentStamp: 'rbc 9/1/2011 22:08' prior: 0!
A stock message, usually subclassed.
!

VSAMessage subclass: #VSAAlert
	instanceVariableNames: 'actuality capability'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'RBC-VSA'!

!VSAAlert commentStamp: 'rbc 11/30/2011 19:52' prior: 0!
A class for alerts sent by an Operation to Manager.  The life cycle of an alert consists of 3 major steps.

1) Alerts are generated by VSA>>mark: using the specified class, usually VSAAlert.  They are stored in the alerts SharedQueue inside the operation.  The threshold for generating an alert is specified by the VSA alertVarience instance variable.

2) VSAManager>>getAlerts is called by the VSAManager>>sendManagerReport: method.  It retrieves and returns all alerts, for all operations in its opsCollection.  This also has the effect of clearing the alerts from the subordinate operations alerts queue.  The alerts collection is stored in the VSAManagerReport message variable.  VSAManager>>sendManagerReport finishes by placing the VSAManagerReport in the VSAManager outBox SharedQueue.

3) The VSAManagerReport is retrived from the VSAManager outBox by VSA>>moveMessages.  It is copied to both the inBox for myExecutive (a VSAExecutive) and myPlanner (a VSAPlanner).

Once myExecutive and myPlanner have the VSAManagerReport, they can use the achievement data and alerts in the report for application specific purposes.
!

!VSAAlert methodsFor: 'accessing' stamp: 'rbc 6/1/2011 21:58'!
actuality
	"Get the actuality."
	^actuality! !

!VSAAlert methodsFor: 'accessing' stamp: 'rbc 6/1/2011 21:50'!
actuality: aNumber
	"Set the actuality"
	actuality := aNumber! !

!VSAAlert methodsFor: 'accessing' stamp: 'rbc 6/1/2011 21:50'!
capability
	"Get the capability"
	^capability! !

!VSAAlert methodsFor: 'accessing' stamp: 'rbc 6/1/2011 21:50'!
capability: aNumber
	"Set the capability"
	capability := aNumber! !

VSAMessage subclass: #VSAManagerReport
	instanceVariableNames: 'actuality capability latency performance'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'RBC-VSA'!

!VSAManagerReport commentStamp: 'rbc 11/30/2011 19:54' prior: 0!
A report sent to Planner and Executive.  It contains achievement data and alerts for subordinate operations.
!

!VSAManagerReport methodsFor: 'accessing' stamp: 'rbc 9/8/2011 10:27'!
actuality
	"Get the actuality"
	^actuality! !

!VSAManagerReport methodsFor: 'accessing' stamp: 'rbc 9/8/2011 10:27'!
actuality: aNumber
	"Set the actuality"
	actuality := aNumber! !

!VSAManagerReport methodsFor: 'accessing' stamp: 'rbc 9/8/2011 10:29'!
capability
	"Get the capability"
	^capability! !

!VSAManagerReport methodsFor: 'accessing' stamp: 'rbc 9/8/2011 10:28'!
capability: aNumber
	"Set the capability"
	capability := aNumber! !

!VSAManagerReport methodsFor: 'accessing' stamp: 'rbc 5/30/2011 17:39'!
latency
	"Get the latency"
	^latency.
	! !

!VSAManagerReport methodsFor: 'accessing' stamp: 'rbc 5/30/2011 17:39'!
latency: aNumber
	"Set the latency"
	latency := aNumber.
	! !

!VSAManagerReport methodsFor: 'accessing' stamp: 'rbc 5/30/2011 17:40'!
performance
	"Get the performance"
	^performance.
	! !

!VSAManagerReport methodsFor: 'accessing' stamp: 'rbc 5/30/2011 17:40'!
performance: aNumber
	"Set the performance"
	performance := aNumber.
	! !

!VSAMessage class methodsFor: 'instance creation' stamp: 'rbc 9/1/2011 21:54'!
new
	^super new setTimeStamp.! !

!VSAMessage methodsFor: 'accessing' stamp: 'rbc 9/1/2011 21:54'!
message
	"Get the message"
	^message! !

!VSAMessage methodsFor: 'accessing' stamp: 'rbc 9/1/2011 21:54'!
message: anObject
	"Set the message"
	message := anObject! !

!VSAMessage methodsFor: 'accessing' stamp: 'rbc 9/1/2011 21:54'!
setTimeStamp
	"Set the time stamp"
	timeStamp := Date dateAndTimeNow! !

!VSAMessage methodsFor: 'accessing' stamp: 'rbc 9/1/2011 21:54'!
timeStamp
	"Get the time stamp"
	^timeStamp! !

!VSAMessage methodsFor: 'accessing' stamp: 'rbc 11/24/2011 10:00'!
vName
	"Get the name of the sending VSA"
	^vName! !

!VSAMessage methodsFor: 'accessing' stamp: 'rbc 11/24/2011 10:00'!
vName: aString
	"Set the name of the sending VSA"
	vName := aString! !

VSAMessage subclass: #VSAOpsMessage
	instanceVariableNames: 'to'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'RBC-VSA'!

!VSAOpsMessage commentStamp: 'rbc 11/30/2011 20:09' prior: 0!
A message, sent between operations.  It is used for operations needing to send objects between one another. 

Delivery of ops messages is done by VSAManager>>supervisorMoveOpsMessages, usually called by VSA>>moveMessages.  Messages are sent via an operations mySupervisor, typically a VSASupervisor.  

VSASupervisor can be subclassed to add application specific anti-oscillation checks to inter-operation messages.  mySupervisor is created by VSA>>createVSA.  Subclassing the VSA allows substituting the class used for mySupervisor, along with myExecutive and myPlanner.
!

!VSAOpsMessage methodsFor: 'accessing' stamp: 'rbc 7/14/2011 20:27'!
to
	"Get the operation number that the message is addressed to"
	^to! !

!VSAOpsMessage methodsFor: 'accessing' stamp: 'rbc 7/14/2011 20:27'!
to: aNumber
	"Set the operation number that the message is addressed to"
	to := aNumber! !

VSAMessage subclass: #VSAPlannerMessage
	instanceVariableNames: 'potentiality'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'RBC-VSA'!

!VSAPlannerMessage commentStamp: 'rbc 11/30/2011 21:17' prior: 0!
A class for sending planning messages from VSAPlanner or its subclass. It is sent from myPlanner to myExecutive (VSAExecutive) and myManager (VSAManager) by VSA>>moveMessages. Planner potentiality calculations are application specific. The message instance variable can be used to ship any kind of object.
!

!VSAPlannerMessage methodsFor: 'accessing' stamp: 'rbc 9/8/2011 10:11'!
potentiality
	"Get the potentiality"
	^potentiality! !

!VSAPlannerMessage methodsFor: 'accessing' stamp: 'rbc 9/8/2011 10:10'!
potentiality: aNumber
	"Set the potentiality"
	potentiality := aNumber! !

VSAMessage subclass: #VSAPolicyMessage
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'RBC-VSA'!

!VSAPolicyMessage commentStamp: 'rbc 11/30/2011 21:19' prior: 0!
A policy message, sent by myExecutive.  It is sent to myPlanner (VSAPlanner) and myManager (VSAManager) by VSA>>moveMessages. The message instance variable can be used to ship any kind of object.
!

Object subclass: #VSAPlanner
	instanceVariableNames: 'inBox outBox'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'RBC-VSA'!

!VSAPlanner commentStamp: 'rbc 7/28/2011 08:20' prior: 0!
A VSA planner class, system 4.

-Responsible for looking outwards to the environment
-Communicates with Manager and Executive
-Determines system potentiality!

!VSAPlanner class methodsFor: 'instance creation' stamp: 'rbc 5/30/2011 16:23'!
new
	"Set up the Planner"
	^super new createPlanner.
	! !

!VSAPlanner methodsFor: 'initialization' stamp: 'rbc 5/30/2011 16:21'!
createPlanner
	"Planner startup"
	inBox := SharedQueue new.
	outBox := SharedQueue new.
! !

!VSAPlanner methodsFor: 'accessing' stamp: 'rbc 5/30/2011 16:55'!
inBox
	"Read a message from the in box"
	^inBox nextOrNil.
	
	! !

!VSAPlanner methodsFor: 'accessing' stamp: 'rbc 5/30/2011 16:55'!
inBox: anObject
	"Add a message object to the in box"
	inBox nextPut: anObject
	! !

!VSAPlanner methodsFor: 'accessing' stamp: 'rbc 5/30/2011 16:54'!
outBox
	"Read a message from the out box"
	^outBox nextOrNil.
	
	! !

!VSAPlanner methodsFor: 'accessing' stamp: 'rbc 5/30/2011 16:55'!
outBox: anObject
	"Add a message object to the out box"
	outBox nextPut: anObject
	! !

!VSAPlanner methodsFor: 'accessing' stamp: 'rbc 12/23/2011 11:18'!
outBoxSize
	"Determine the size of the out box"
	^outBox size
! !

Object subclass: #VSASupervisor
	instanceVariableNames: 'inBox outBox'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'RBC-VSA'!

!VSASupervisor commentStamp: 'rbc 11/30/2011 21:22' prior: 0!
A supervisor (system 2), paired with an operation (system 1).

-Provides conduits for communications between multiple operations
-Provides Manager with conduits to monitor and coordinate system operation activities
-Communicates with paired Operation, other supervisors and Manager

VSASupervisor can be subclassed to add application specific anti-oscillation checks to inter-operation messages.  mySupervisor is created by VSA>>createVSA.  Subclassing the VSA allows substituting the class used for mySupervisor, along with myExecutive and myPlanner.
!

!VSASupervisor class methodsFor: 'instance creation' stamp: 'rbc 7/12/2011 17:21'!
new
	"Set up the Supervisor"
	^super new createSupervisor.
	! !

!VSASupervisor methodsFor: 'initialization' stamp: 'rbc 7/12/2011 17:21'!
createSupervisor
	"Supervisor startup"
	inBox := SharedQueue new.
	outBox := SharedQueue new.
! !

!VSASupervisor methodsFor: 'accessing' stamp: 'rbc 7/12/2011 17:20'!
inBox
	"Read a message from the in box"
	^inBox nextOrNil.
	
	! !

!VSASupervisor methodsFor: 'accessing' stamp: 'rbc 7/12/2011 17:20'!
inBox: anObject
	"Add a message object to the in box"
	inBox nextPut: anObject
	! !

!VSASupervisor methodsFor: 'accessing' stamp: 'rbc 7/12/2011 17:20'!
outBox
	"Read a message from the out box"
	^outBox nextOrNil.
	
	! !

!VSASupervisor methodsFor: 'accessing' stamp: 'rbc 7/12/2011 17:20'!
outBox: anObject
	"Add a message object to the out box"
	outBox nextPut: anObject
	! !

!VSASupervisor methodsFor: 'accessing' stamp: 'rbc 12/17/2011 00:59'!
outBoxSize
	"Determine the size of the out box"
	^outBox size
! !
